==> skypager-project-2.54.0.tgz: package/src/framework.js <==
//if (!process.env.SKYPAGER_DIST) { require('./environment')() }

import Collection from './collection'
import Registry from './registry'
import Project from './project'

import * as util from './util'
import * as Assets from './assets'
import * as Helpers from './helpers'

import { dirname, resolve, join, basename, extname } from 'path'

import defaults from 'lodash/defaultsDeep'
import get from 'lodash/get'
import isEmpty from 'lodash/isEmpty'

import pathExists from 'path-exists'

GLOBAL.invariant = require('invariant')

// temp
delete (Assets.default)
delete (Helpers.default)
delete (Project.default)

const ProjectCache = {
  projects: {  }
}

const pkg = require('../package.json')

class Framework {

  static Version = pkg.version;
  static Name = pkg.name;

  constructor (options = {}, initializer) {
    options = defaults(options, {
      type: 'framework',
      key: 'skypager',
      transforms: [],
      root: __dirname
    })

    this.root = options.root
    this.type = options.type
    this.key = options.key

    util.hide.property(this, 'createStamp', Math.floor(Date.now() / 100))
    util.hide.property(this, 'registryId', `${this.key}-${ this.type }:${ this.createStamp }`)

    //util.hide.getter(this, 'manifest', require('../package.json'))

    this.registries = Registry.buildAll(this, Helpers, {root})

    this.transforms = []

    util.hide.property(this, 'enabledPlugins', [])
    util.hide.property(this, '_loadedPlugins', [])

    this.discoverPlugins()

    if (typeof (initializer) === 'function') {
      initializer(this)
    }

    //require('./environment')(this.root)
  }

  get version() {
    return this.constructor.Version
  }

  get moduleName () {
     return this.constructor.Name
  }

  get isNodeModule() {
    return basename(this.moduleRoot) === 'node_modules'
  }

  get isLocalPackage() {
    return basename(this.moduleRoot) === 'packages'
  }

  get isCompiled () {
    return basename(this.root) === 'lib'
  }

  get moduleRoot () {
     return dirname(join(this.root, '..'))
  }

  get peerModules () {
    return require('fs').readdirSync(this.moduleRoot)
      .filter(name => name && name.match(this.key))
  }

  discoverPlugins() {
    let projectManifest = attemptRequire(
      join(process.env.PWD, 'package.json')
    )

    let pluginIds = []

    if (projectManifest) {
      pluginIds = pluginIds.concat( get(projectManifest, `skypager.plugins`) || [])
    }

    if (this.key !== 'skypager') {
      pluginIds = pluginIds.concat( get(projectManifest, `${ this.key }.plugins`) || [])
    }

    eagerLoadProjectPlugins(this, pluginIds)

    // this will search nodes require paths for plugins
    // holding off for now
    //eagerLoadFoundPlugins(this)
  }

  get allProjects() {
    return Object.values(ProjectCache.projects)
  }

  get projectCacheKeys() {
     return Object.keys(ProjectCache.projects)
  }

  clearProjectCache() {
     this.projectCacheKeys.forEach(key => delete(ProjectCache.projects[key]))
  }
  /**
  * Create a project loader for the specified path.
  * Returns a function you can execute when you want the project loaded.
  */
  create (pathToProject, options = {}) {
    let skypager = this

    return function(opts = {}) {
      return new skypager.load(pathToProject, Object.assign(options, opts))
    }
  }

  get isBabelRegistered() {
    if (this.babelRegistered) { return true }

    if ( require.extensions && require.extensions['.es6'] ) {
      return true
    }
  }

  maybeBabel (options = {}) {
    try { require('babel-polyfill')  } catch(error) {}

    if(!this.isBabelRegistered) {
      try {
        require('babel-register')({ presets:['skypager'] })
      } catch(error) {
        console.log('There was an error trying to babel-register', error)
        return this.babelRegistered = false
      }
    }

    return this.babelRegistered = true
  }

  /**
  * Load a project in the specified path immediately.
  *
  */
  load (projectFile = process.env.PWD, options = {}) {
    let skypager = this

    skypager.maybeBabel()

    if (typeof projectFile === 'object') {
      if (isEmpty(options)) {
        options = projectFile
      }

      projectFile = options.root || process.env.PWD
    }

    if (!options.fresh && ProjectCache.projects[projectFile]) {
       return ProjectCache.projects[projectFile]
    }

    let root = projectFile

    if(projectFile.match(/\.(js|json)$/i)) {
      root = dirname(projectFile)
    }

    if (!options.manifest) {
      try {
        options.manifest = require( join(root, 'package.json'))
      } catch(error) {
        console.log('Error loading manifest', error.message, join(root,'package.json'))
      }
    }

    defaults(options, {
      manifest: {
        skypager: {
          plugins:[]
        }
      }
    }, {
      [this.key]: {
        plugins: []
      }
    })

    let project = new this.Project(projectFile, options)

    util.hide.getter(project, '_framework', () => this)

    let transforms = this.transforms.filter(t => typeof t === 'function')

    transforms.forEach(transform => transform(project, this))

    return ProjectCache.projects[projectFile] = this.wrap(project)
  }

  wrap(project) {
    return project
  }

  get loadedProjects() {
    return Object.values(ProjectCache.projects)
  }

  use (plugins, options = {}) {
    if (typeof plugins === 'string') {
      plugins = [plugins]
    }

    plugins.forEach(plugin => {
      let pluginConfig = this.plugins.lookup(plugin)

      if (pluginConfig && pluginConfig.api && pluginConfig.api.modify) {
        pluginConfig.api.modify(this)
      } else {
        if (pluginConfig && typeof pluginConfig.api === 'function') {
          pluginConfig.api.call(this, this, pluginConfig)
        }
      }

      this.enabledPlugins.push(plugin)
    })
  }

  /**
   * Load a plugin by name, or by passing the plugins loader function
   *
   * @param {String||Function} moduleId a required plugin or a requireable path to that plugin
   *  will attempt to find the plugin by attaching the skypager-plugin- prefix as well.
  */
  loadPlugin(request) {
    let loader

    if (typeof request === 'string') {
      loader = attemptRequires(
        request,
        `skypager-plugin-${request}`,
        `${ this.key }-plugin-${ request }`
      )
    }

    /*
    if(this._loadedPlugins.indexOf((loader || request).name) >= 0) {
      return
    }
    */

    try {
      this.plugins.runLoader(loader || request)
      //this._loadedPlugins.push((loader || request).name)
      return true
    } catch(error) {
      return false
    }
  }

  get actions () {
    return this.registries.actions
  }

  get contexts () {
    return this.registries.contexts
  }

  get exporters () {
    return this.registries.exporters
  }

  get importers () {
    return this.registries.importers
  }

  get models () {
    return this.registries.models
  }

  get plugins () {
    return this.registries.plugins
  }

  get renderers () {
    return this.registries.renderers
  }

  get util () {
    return util
  }
}

function eagerLoadProjectPlugins(framework, list) {
  let requestedPlugins = list.filter(item => typeof item === 'string' && item.length > 0)

  requestedPlugins.map(
    request => attemptRequires( request, `skypager-plugin-${ request }`, `${ framework.key }-plugin-${ request }`)
  )
  .map(foundPlugin => foundPlugin && attemptRequire(foundPlugin))
  .map(
    loader => loader && loader.default ? loader.default : loader
  ).filter(
    loader => typeof loader === 'function'
  ).map(
    loader => framework.loadPlugin(loader)
  ).filter(result => !!result)
}

function eagerLoadFoundPlugins(framework) {
  let checkFolders = []

  try {
    checkFolders = require.cache[module.id].paths
  } catch(error) {
    checkFolders = []
  }

  checkFolders = checkFolders.filter(dir => pathExists.sync(dir))

  if (framework.key !== 'skypager') {
    checkFolders.forEach(moduleFolder => {
      getModulesMatchingPrefix(moduleFolder, 'skypager-plugin')
    })
  }

  checkFolders.forEach(moduleFolder => {
    getModulesMatchingPrefix(moduleFolder, framework.key + '-plugin').forEach(plugin => framework.loadPlugin(plugin))
  })
}

function attemptRequires(...guesses) {
  try { return guesses.unique().find(attemptRequire) } catch(error) {
  }
}

function attemptRequire(moduleId) {
  try {
    return require(moduleId)
  } catch(error) {
  }
}

function getModulesMatchingPrefix(dir, prefix) {
  var fs = require('fs')
  var path = require('path')
  var dirs = []
  var files = fs.readdirSync(dir)

  files.forEach(function(file) {
    var fullPath = path.join(dir, file)
    var stat = fs.statSync(fullPath)
    if (stat.isDirectory() && file.startsWith(prefix)) {
      dirs.push(file)
    }
  })

  return dirs
}


module.exports = Framework

==> skypager-project-2.59.4.tgz: package/src/framework.js <==
//if (!process.env.SKYPAGER_DIST) { require('./environment')() }

import Collection from './collection'
import Registry from './registry'
import Project from './project'

import * as util from './util'
import * as Assets from './assets'
import * as Helpers from './helpers'

import { dirname, resolve, join, basename, extname } from 'path'

import defaults from 'lodash/defaultsDeep'
import get from 'lodash/get'
import isEmpty from 'lodash/isEmpty'

import pathExists from 'path-exists'

GLOBAL.invariant = require('invariant')

// temp
delete (Assets.default)
delete (Helpers.default)
delete (Project.default)

const ProjectCache = {
  projects: {  }
}

const pkg = require('../package.json')

class Framework {

  static Version = pkg.version;
  static Name = pkg.name;

  constructor (options = {}, initializer) {
    options = defaults(options, {
      type: 'framework',
      key: 'skypager',
      transforms: [],
      root: __dirname
    })

    this.root = options.root
    this.type = options.type
    this.key = options.key

    util.hide.property(this, 'createStamp', Math.floor(Date.now() / 100))
    util.hide.property(this, 'registryId', `${this.key}-${ this.type }:${ this.createStamp }`)

    //util.hide.getter(this, 'manifest', require('../package.json'))

    this.registries = Registry.buildAll(this, Helpers, {root})

    this.transforms = []

    util.hide.property(this, 'enabledPlugins', [])
    util.hide.property(this, '_loadedPlugins', [])

    this.discoverPlugins()

    if (typeof (initializer) === 'function') {
      initializer(this)
    }

    //require('./environment')(this.root)
  }

  get version() {
    return this.constructor.Version
  }

  get moduleName () {
     return this.constructor.Name
  }

  get isNodeModule() {
    return basename(this.moduleRoot) === 'node_modules'
  }

  get isLocalPackage() {
    return basename(this.moduleRoot) === 'packages'
  }

  get isCompiled () {
    return basename(this.root) === 'lib'
  }

  get moduleRoot () {
     return dirname(join(this.root, '..'))
  }

  get peerModules () {
    return require('fs').readdirSync(this.moduleRoot)
      .filter(name => name && name.match(this.key))
  }

  discoverPlugins() {

    if (!pathExists.sync(join(process.env.PWD, 'package.json'))) {
      return
    }

    let projectManifest = require(join(process.env.PWD, 'package.json'))
    let pluginIds = []

    if (projectManifest) {
      pluginIds = pluginIds.concat( get(projectManifest, `skypager.plugins`) || [])
    }

    if (this.key !== 'skypager') {
      pluginIds = pluginIds.concat( get(projectManifest, `${ this.key }.plugins`) || [])
    }

    eagerLoadProjectPlugins(this, pluginIds)

    // this will search nodes require paths for plugins
    // holding off for now
    //eagerLoadFoundPlugins(this)
  }

  get allProjects() {
    return Object.values(ProjectCache.projects)
  }

  get projectCacheKeys() {
     return Object.keys(ProjectCache.projects)
  }

  clearProjectCache() {
     this.projectCacheKeys.forEach(key => delete(ProjectCache.projects[key]))
  }
  /**
  * Create a project loader for the specified path.
  * Returns a function you can execute when you want the project loaded.
  */
  create (pathToProject, options = {}) {
    let skypager = this

    return function(opts = {}) {
      return new skypager.load(pathToProject, Object.assign(options, opts))
    }
  }

  get isBabelRegistered() {
    if (this.babelRegistered) { return true }

    if ( require.extensions && require.extensions['.es6'] ) {
      return true
    }
  }

  maybeBabel (options = {}) {
    try { require('babel-polyfill')  } catch(error) {}

    if(!this.isBabelRegistered) {
      try {
        require('babel-register')({ presets:['skypager'] })
      } catch(error) {
        console.log('There was an error trying to babel-register', error)
        return this.babelRegistered = false
      }
    }

    return this.babelRegistered = true
  }

  /**
  * Load a project in the specified path immediately.
  *
  */
  load (projectFile = process.env.PWD, options = {}) {
    let skypager = this

    skypager.maybeBabel()

    if (typeof projectFile === 'object') {
      if (isEmpty(options)) {
        options = projectFile
      }

      projectFile = options.root || process.env.PWD
    }

    if (!options.fresh && ProjectCache.projects[projectFile]) {
       return ProjectCache.projects[projectFile]
    }

    let root = projectFile

    if(projectFile.match(/\.(js|json)$/i)) {
      root = dirname(projectFile)
    }

    if (!options.manifest) {
      try {
        options.manifest = require( join(root, 'package.json'))
      } catch(error) {
        console.log('Error loading manifest', error.message, join(root,'package.json'))
      }
    }

    defaults(options, {
      manifest: {
        skypager: {
          plugins:[]
        }
      }
    }, {
      [this.key]: {
        plugins: []
      }
    })

    let project = new this.Project(projectFile, options)

    util.hide.getter(project, '_framework', () => this)

    let transforms = this.transforms.filter(t => typeof t === 'function')

    transforms.forEach(transform => transform(project, this))

    return ProjectCache.projects[projectFile] = this.wrap(project)
  }

  wrap(project) {
    return project
  }

  get loadedProjects() {
    return Object.values(ProjectCache.projects)
  }

  use (plugins, options = {}) {
    if (typeof plugins === 'string') {
      plugins = [plugins]
    }

    plugins.forEach(plugin => {
      let pluginConfig = this.plugins.lookup(plugin)

      if (pluginConfig && pluginConfig.api && pluginConfig.api.modify) {
        pluginConfig.api.modify(this)
      } else {
        if (pluginConfig && typeof pluginConfig.api === 'function') {
          pluginConfig.api.call(this, this, pluginConfig)
        }
      }

      this.enabledPlugins.push(plugin)
    })
  }

  /**
   * Load a plugin by name, or by passing the plugins loader function
   *
   * @param {String||Function} moduleId a required plugin or a requireable path to that plugin
   *  will attempt to find the plugin by attaching the skypager-plugin- prefix as well.
  */
  loadPlugin(request) {
    let loader

    if (typeof request === 'string') {
      loader = attemptRequires(
        request,
        `skypager-plugin-${request}`,
        `${ this.key }-plugin-${ request }`
      )
    }

    /*
    if(this._loadedPlugins.indexOf((loader || request).name) >= 0) {
      return
    }
    */

    try {
      this.plugins.runLoader(loader || request)
      //this._loadedPlugins.push((loader || request).name)
      return true
    } catch(error) {
      return false
    }
  }

  get actions () {
    return this.registries.actions
  }

  get contexts () {
    return this.registries.contexts
  }

  get exporters () {
    return this.registries.exporters
  }

  get importers () {
    return this.registries.importers
  }

  get models () {
    return this.registries.models
  }

  get plugins () {
    return this.registries.plugins
  }

  get renderers () {
    return this.registries.renderers
  }

  get util () {
    return util
  }
}

function eagerLoadProjectPlugins(framework, list) {
  let requestedPlugins = list.filter(item => typeof item === 'string' && item.length > 0)

  requestedPlugins
  .map(
    request => util.findPackageSync(request) || request
  )
  .map(
    request => attemptRequires( request, `skypager-plugin-${ request }`, `${ framework.key }-plugin-${ request }`)
  )
  .map(foundPlugin => foundPlugin && attemptRequire(foundPlugin))
  .map(
    loader => loader && loader.default ? loader.default : loader
  ).filter(
    loader => typeof loader === 'function'
  ).map(
    loader => framework.loadPlugin(loader)
  ).filter(result => !!result)
}

function eagerLoadFoundPlugins(framework) {
  let checkFolders = []

  try {
    checkFolders = require.cache[module.id].paths
  } catch(error) {
    checkFolders = []
  }

  checkFolders = checkFolders.filter(dir => pathExists.sync(dir))

  if (framework.key !== 'skypager') {
    checkFolders.forEach(moduleFolder => {
      getModulesMatchingPrefix(moduleFolder, 'skypager-plugin')
    })
  }

  checkFolders.forEach(moduleFolder => {
    getModulesMatchingPrefix(moduleFolder, framework.key + '-plugin').forEach(plugin => framework.loadPlugin(plugin))
  })
}

function attemptRequires(...guesses) {
  try { return guesses.unique().find(attemptRequire) } catch(error) { }
}

function attemptRequire(moduleId) {
  try {
    return require(moduleId)
  } catch(error) {
  }
}

function getModulesMatchingPrefix(dir, prefix) {
  var fs = require('fs')
  var path = require('path')
  var dirs = []
  var files = fs.readdirSync(dir)

  files.forEach(function(file) {
    var fullPath = path.join(dir, file)
    var stat = fs.statSync(fullPath)
    if (stat.isDirectory() && file.startsWith(prefix)) {
      dirs.push(file)
    }
  })

  return dirs
}


module.exports = Framework
